<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\ObjectManagerInterface;
use Magento\PageBuilder\Model\Dom\HtmlDocumentFactory;
use Pelago\Emogrifier\CssInliner;
use Symfony\Component\CssSelector\Exception\ParseException;
use function class_exists;

/**
 *
 */
class PageBuilderConverter
{

    /**
     * @var HtmlDocumentFactory | null
     */
    private $documentFactory;

    /**
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        ObjectManagerInterface $objectManager
    ){
        if (class_exists(HtmlDocumentFactory::class)) {
            $this->documentFactory = $objectManager->get(HtmlDocumentFactory::class);
        }
    }

    /**
     * @param $data
     * @return string
     * @throws ParseException
     */
    public function convert($data)
    {
        if (!$this->documentFactory) {
            return $data;
        }
        $document = $this->documentFactory->create(['document' => $data]);
        $document->querySelector('body')->setAttribute('id', 'html-body');
        return CssInliner::fromHtml($document->getContents())->inlineCss()->renderBodyContent();
    }

}
